@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Warp Networks S.L., DBS Servicios Informaticos S.L.
+# Copyright (C) 2008-2010 eBox Technologies S.L.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2, as
@@ -71,13 +71,12 @@ sub _user {
 	$self->_requireParam('quota', __('Disk quota limit'));
 	my $user = $self->param('user');
 	my $active = $self->param('active');
-	my $uid = $users->userInfo($user)->{'uid'};
 
 	$self->{redirect} = "UsersAndGroups/User?username=$user";
 
 	$smbldap->setUserSharing($user, $active);
+	$smbldap->setUserQuota($user, $self->param('quota'));
 	$smb->setAdminUser($user, $self->param('is_admin'));
-	$smb->setUserQuota($uid, $self->param('quota'));
 }
 
 sub _process($) {
