@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Warp Networks S.L., DBS Servicios Informaticos S.L.
+# Copyright (C) 2008-2010 eBox Technologies S.L.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2, as
@@ -22,10 +22,10 @@ use base qw(EBox::Module::Service EBox::LdapModule EBox::FirewallObserver
             EBox::Report::DiskUsageProvider EBox::Model::CompositeProvider
             EBox::Model::ModelProvider EBox::LogObserver);
 
-use EBox::Sudo qw( :all );
+use EBox::Sudo;
 use EBox::Global;
 use EBox::Service;
-use EBox::SambaLdapUser qw(PROFILESPATH);
+use EBox::SambaLdapUser;
 use EBox::UsersAndGroups;
 use EBox::Network;
 use EBox::SambaFirewall;
@@ -41,6 +41,7 @@ use EBox::Exceptions::External;
 use EBox::Gettext;
 use EBox::Config;
 use EBox::Model::ModelManager;
+use EBox::DBEngineFactory;
 
 
 use File::Slurp qw(read_file write_file);
@@ -50,7 +51,6 @@ use Sys::Hostname;
 
 use constant SMBCONFFILE          => '/etc/samba/smb.conf';
 use constant CLAMAVSMBCONFFILE    => '/etc/samba/vscan-clamav.conf';
-use constant LIBNSSLDAPFILE       => '/etc/ldap.conf';
 use constant SMBLDAPTOOLBINDFILE  => '/etc/smbldap-tools/smbldap_bind.conf';
 use constant SMBLDAPTOOLBINDFILE_MASK => '0600';
 use constant SMBLDAPTOOLBINDFILE_UID => '0';
@@ -62,12 +62,12 @@ use constant MAXNETBIOSLENGTH     => 15;
 use constant MAXWORKGROUPLENGTH   => 32;
 use constant MAXDESCRIPTIONLENGTH => 255;
 use constant SMBPORTS => qw(137 138 139 445);
-use constant NMBD_PID => '/var/run/samba/nmbd.pid';
-use constant SMBD_PID => '/var/run/samba/smbd.pid';
+use constant NETLOGONDIR          => '/home/samba/netlogon';
+use constant NETLOGONSCRIPT       => 'logon.bat';
+use constant NETLOGONDEFAULTSCRIPT=> 'zentyal-logon.bat';
 
 
 use constant FIX_SID_PROGRAM => '/usr/share/ebox-samba/ebox-fix-sid';
-use constant QUOTA_PROGRAM => '/usr/share/ebox-samba/ebox-samba-quota';
 
 sub _create
 {
@@ -80,6 +80,31 @@ sub _create
     return $self;
 }
 
+# Method: bootDepends
+#
+#     Samba depends on CUPS only if printers module enabled.
+#
+# Overrides:
+#
+#     <EBox::Module::Base::depends>
+#
+sub bootDepends
+{
+    my ($self) = @_;
+
+    my $dependsList = $self->depends();
+
+    my $module = 'printers';
+    if (EBox::Global->modExists($module)) {
+        my $printers = EBox::Global->modInstance($module);
+        if ($printers->isEnabled()) {
+            push (@{$dependsList}, $module);
+        }
+    }
+
+    return $dependsList;
+}
+
 # Method: actions
 #
 #	Override EBox::Module::Service::actions
@@ -88,21 +113,21 @@ sub actions
 {
     return [
     {
-        'action' => __('Create Samba home directory for users and groups'),
-        'reason' => __('eBox will create the home directories for Samba ' .
-            'users and groups under /home/samba.'),
+        'action' => __('Create Samba home directory for shares and groups'),
+        'reason' => __('Zentyal will create the directories for Samba ' .
+                       'shares and groups under /home/samba.'),
         'module' => 'samba'
     },
     {
         'action' => __('Add LDAP schemas'),
-        'reason' => __('eBox will add two LDAP schemas to the LDAP directory:' .
-            ' samba and ebox.'),
+        'reason' => __('Zentyal will add two LDAP schemas to the LDAP directory:' .
+                       ' samba and ebox.'),
         'module' => 'samba'
     },
     {
         'action' => __('Set Samba LDAP admin dn password'),
-        'reason' => __('eBox will configure Samba to use the LDAP admin dn ' .
-            'password.'),
+        'reason' => __('Zentyal will configure Samba to use the LDAP admin dn ' .
+                       'password.'),
         'module' => 'samba'
     },
     ];
@@ -116,41 +141,30 @@ sub usedFiles
 {
     return [
     {
-        'file' => '/etc/samba/smb.conf',
-            'reason' => __('To set up Samba according to your configuration'),
+        'file' => SMBCONFFILE,
+            'reason' => __('To set up Samba according to your configuration.'),
         'module' => 'samba'
     },
     {
-        'file' => '/etc/smbldap-tools/smbldap.conf',
+        'file' => SMBLDAPTOOLCONFFILE,
         'reason' => __('To set up smbldap-tools according to your' .
-            ' configuration'),
+            ' configuration.'),
         'module' => 'samba'
     },
     {
-        'file' => '/etc/smbldap-tools/smbldap_bind.conf',
+        'file' => SMBLDAPTOOLBINDFILE,
         'reason' => __('To set up smbldap-tools according to your LDAP' .
-            ' configuration'),
-        'module' => 'samba'
-    },
-    {
-        'file' => '/etc/nsswitch.conf',
-        'reason' => __('To make NSS use LDAP resolution for user and group '.
-            'accounts. Needed for Samba PDC configuration.'),
-        'module' => 'samba'
-    },
-    {
-        'file' => '/etc/ldap.conf',
-        'reason' => __('To let NSS know how to access LDAP accounts'),
+            ' configuration.'),
         'module' => 'samba'
     },
     {
         'file' => '/etc/fstab',
-        'reason' => __('To add quota support to /home partition'),
+        'reason' => __('To add quota support to /home partition.'),
         'module' => 'samba'
     },
     {
-        'file' => '/etc/samba/vscan-clamav.conf',
-        'reason' => __('To set the antivirus settings for Samba'),
+        'file' => CLAMAVSMBCONFFILE,
+        'reason' => __('To set the antivirus settings for Samba.'),
         'module' => 'samba'
     },
     ];
@@ -168,7 +182,7 @@ sub enableActions
 
     my $users = EBox::Global->modInstance('users');
 
-    root(EBox::Config::share() . '/ebox-samba/ebox-samba-enable');
+    EBox::Sudo::root(EBox::Config::share() . '/ebox-samba/ebox-samba-enable');
 }
 
 # Method: modelClasses
@@ -217,7 +231,13 @@ sub compositeClasses
 
 # Method: shares
 #
-#   It returns the custom shares added by the user.
+#   It returns the custom shares 8
+#
+# Parameters:
+#
+#     all - return all shares regardless of their permission
+#           level. Otherwise shares without permssions or guset access are
+#           ignored. Default: false
 #
 # Returns:
 #
@@ -234,7 +254,7 @@ sub compositeClasses
 #   validUsers - readOnly + readWrite + administrators
 sub shares
 {
-    my ($self) = @_;
+    my ($self, $all) = @_;
     my $shares = $self->model('SambaShares');
     my @shares;
 
@@ -255,6 +275,8 @@ sub shares
         }
         $shareConf->{'path'} .= $path->value();
 
+        $shareConf->{'guest'} = $row->elementByName('guest')->value();
+
         for my $subId (@{$row->subModel('access')->ids()}) {
             my $subRow = $row->subModel('access')->row($subId);
             my $userType = $subRow->elementByName('user_group');
@@ -275,7 +297,10 @@ sub shares
             }
         }
 
-        next unless (@readOnly or @readWrite or @administrators);
+        if (not $all) {
+            next unless (@readOnly or @readWrite or @administrators
+                         or $shareConf->{'guest'});
+        }
 
         $shareConf->{'readOnly'} = join (', ', @readOnly);
         $shareConf->{'readWrite'} = join (', ', @readWrite);
@@ -397,9 +422,6 @@ sub _exposedMethods
 }
 
 
-
-
-
 # return interface upon samba should listen
 # XXX this is a quick fix for this version. See #529
 sub sambaInterfaces
@@ -410,9 +432,25 @@ sub sambaInterfaces
     my $global = EBox::Global->getInstance();
 
     my $net = $global->modInstance('network');
-    my $internalIfaces = $net->InternalIfaces;
-    foreach my $iface (@{ $internalIfaces }) {
+
+    my $listen_external = EBox::Config::configkey('listen_external');
+
+    my $netIfaces;
+    if ($listen_external eq 'yes') {
+        $netIfaces = $net->allIfaces;
+    } else {
+        $netIfaces = $net->InternalIfaces;
+    }
+
+    foreach my $iface (@{ $netIfaces }) {
         push @ifaces, $iface;
+
+        if ( $net->ifaceMethod($iface) eq 'bridged' ) {
+            my $br = $net->ifaceBridge($iface);
+            push (@ifaces, "br$br");
+            next;
+        }
+
         my $vifacesNames = $net->vifaceNames($iface);
         if (defined $vifacesNames) {
             push @ifaces, @{  $vifacesNames };
@@ -476,34 +514,40 @@ sub _setConf
     push(@array, 'roaming' => $self->roamingProfiles());
     push(@array, 'backup_path' => EBox::Config::conf() . '/backups');
     push(@array, 'quarantine_path' => EBox::Config::var() . '/lib/ebox/quarantine');
-    push(@array, 'shares' => $self->shares());
+    my $shares = $self->shares();
+    push(@array, 'shares' => $shares);
+    my $guestShares = 0;
+    foreach my $share (@{$shares}) {
+        if ($share->{'guest'}) {
+            $guestShares = 1;
+            last;
+        }
+    }
+    push(@array, 'guest_shares' => $guestShares);
     push(@array, 'antivirus' => $self->defaultAntivirusSettings());
     push(@array, 'antivirus_exceptions' => $self->antivirusExceptions());
     push(@array, 'recycle' => $self->defaultRecycleSettings());
     push(@array, 'recycle_exceptions' => $self->recycleExceptions());
     push(@array, 'recycle_config' => $self->recycleConfig());
 
+    if ($self->pdc()) {
+        my $logonScript = join('/', NETLOGONDIR, NETLOGONSCRIPT);
+        if (EBox::Sudo::fileTest('-f', $logonScript)) {
+            push(@array, 'logon_script', NETLOGONSCRIPT);
+        }
+        $self->writeConfFile(join('/', NETLOGONDIR, NETLOGONDEFAULTSCRIPT),
+            'samba/logon.bat.mas', \@array);
+    }
 
-    $self->writeConfFile(SMBCONFFILE, "samba/smb.conf.mas", \@array);
+    $self->writeConfFile(SMBCONFFILE, 'samba/smb.conf.mas', \@array);
 
-    $self->writeConfFile(CLAMAVSMBCONFFILE, "samba/vscan-clamav.conf.mas", \@array);
+    $self->writeConfFile(CLAMAVSMBCONFFILE, 'samba/vscan-clamav.conf.mas',
+        \@array);
 
-    root(EBox::Config::share() . '/ebox-samba/ebox-setadmin-pass');
+    EBox::Sudo::root(EBox::Config::share() . '/ebox-samba/ebox-setadmin-pass');
 
     my $users = EBox::Global->modInstance('users');
 
-    @array = ();
-    push(@array, 'basedc'    => $ldapconf->{'dn'});
-    push(@array, 'ldap'     => $ldapconf->{'ldapi'});
-    push(@array, 'binddn'     => $ldapconf->{'rootdn'});
-    push(@array, 'bindpw'    => $ldap->getPassword());
-    push(@array, 'usersdn'   => $users->usersDn);
-    push(@array, 'groupsdn'  => $users->groupsDn);
-    push(@array, 'computersdn' => 'ou=Computers,' . $ldapconf->{'dn'});
-
-    $self->writeConfFile(LIBNSSLDAPFILE, "samba/ldap.conf.mas",
-            \@array);
-
     $self->setSambaLdapToolsConf();
 
     # Set SIDs using the net tool this is neccesary bz if we change the
@@ -517,6 +561,9 @@ sub _setConf
     # Create samba shares
     $self->model('SambaShares')->createDirs();
 
+    # Change group ownership of quarantine_dir to __USERS__
+    my $quarantine_dir = EBox::Config::var() . '/lib/ebox/quarantine';
+    EBox::Sudo::silentRoot("chown root:__USERS__ $quarantine_dir");
 }
 
 
@@ -663,30 +710,33 @@ sub widgets
 {
     return {
         'sharesbyuser' => {
-            'title' => __("Shares by user"),
-                'widget' => \&sharesByUserWidget,
-                'default' => 1
+            'title' => __('Shares by user'),
+            'widget' => \&sharesByUserWidget,
+            'order' => 7,
+            'default' => 1
         },
         'usersbyshare' => {
-            'title' => __("Users by share"),
-                'widget' => \&usersByShareWidget,
-                'default' => 1
+            'title' => __('Users by share'),
+            'widget' => \&usersByShareWidget,
+            'order' => 9,
+            'default' => 1
         }
     };
 }
 
 # Method: _daemons
 #
-#	Override EBox::Module::Service::_daemons
+#       Override EBox::Module::Service::_daemons
 #
 sub _daemons
 {
     return [
         {
-            'name' => 'samba',
-            'type' => 'init.d',
-            'pidfiles' => [SMBD_PID, NMBD_PID],
+            'name' => 'nmbd',
         },
+        {
+            'name' => 'smbd',
+        }
     ];
 }
 
@@ -950,30 +1000,12 @@ sub _checkDescriptionName # (name)
     return 1;
 }
 
-sub _checkQuota # (quota)
-{
-    my ($quota) = @_;
-
-    ($quota =~ /\D/) and return undef;
-    return 1;
-}
-
-sub addPrinter # (resource)
+sub _addPrinter
 {
     my ($self, $name) = @_;
 
-    return if ($self->dir_exists("printers/$name"));
-    $self->set_list("printers/$name/users", "string", []);
-    $self->set_list("printers/$name/groups", "string", []);
-    $self->set_bool("printers/external", undef);
-
-}
-
-sub _addExternalPrinter
-{
-    my ($self, $name) = @_;
-    $self->set_list("printers/$name/users", "string", []);
-    $self->set_list("printers/$name/groups", "string", []);
+    $self->set_list("printers/$name/users", 'string', []);
+    $self->set_list("printers/$name/groups", 'string', []);
     $self->set_bool("printers/$name/external", 1);
 }
 
@@ -985,29 +1017,27 @@ sub printers
     my %external;
     if ($global->modExists('printers')) {
         my $printers = $global->modInstance('printers');
-        %external = map { $_ => 1 } @{$printers->fetchExternalCUPSPrinters()};
+        %external = map { $_ => 'new' } @{$printers->fetchExternalCUPSPrinters()};
     } else {
         return [];
     }
 
     my @printers;
     my $readOnly = $self->isReadOnly();
-    for my $printer (@{$self->array_from_dir("printers")}) {
-        my $name = $printer->{_dir};
-        my $key = "printers/$name/external";
-        my $isExt = $self->get_bool($key);
-        if ($isExt and not exists $external{$name}) {
+    for my $printer (@{$self->array_from_dir('printers')}) {
+        my $name = $printer->{'_dir'};
+        if (exists $external{$name}) {
+            $external{$name} = 'exists';
+        } else {
             $self->delPrinter($name) unless ($readOnly);
             $external{$name} = 'removed';
-        }  elsif ($isExt) {
-            $external{$name} = 'exists';
         }
         push (@printers,  $printer->{'_dir'});
     }
 
     unless ($readOnly) {
-        for my $newPrinter (grep { $external{$_} == 1  } keys %external) {
-            $self->_addExternalPrinter($newPrinter);
+        for my $newPrinter (grep { $external{$_} eq 'new' } keys %external) {
+            $self->_addPrinter($newPrinter);
             push (@printers, $newPrinter);
         }
     }
@@ -1015,21 +1045,36 @@ sub printers
     return [sort @printers];
 }
 
+sub ignorePrinterNotFound
+{
+    my ($self) = @_;
+
+    return $self->get_bool('ignorePrinterNotFound');
+}
+
+sub _printerNotFound
+{
+    my ($self, $printer) = @_;
+
+    unless ($self->ignorePrinterNotFound()) {
+        throw EBox::Exceptions::DataNotFound('data' => 'printer',
+                'value' => $printer);
+    }
+}
+
 sub _addUsersToPrinter # (printer, users)
 {
     my ($self, $printer, $users) = @_;
 
     unless ($self->dir_exists("printers/$printer")) {
-        throw EBox::Exceptions::DataNotFound('data' => 'printer',
-                'value' => $printer);
+        $self->_printerNotFound($printer);
+        return;
     }
 
-
     for my $username (@{$users}) {
         _checkUserExists($username);
     }
 
-
     $self->set_list("printers/$printer/users", "string", $users);
 }
 
@@ -1038,12 +1083,15 @@ sub _addGroupsToPrinter # (printer, groups)
     my ($self, $printer, $groups) = @_;
 
     unless ($self->dir_exists("printers/$printer")) {
-        throw EBox::Exceptions::DataNotFound('data' => 'printer',
-                'value' => $printer);
+        $self->_printerNotFound($printer);
+        return;
     }
 
-    for my $groupname (@{$groups}) {
-        _checkGroupExists($groupname);
+    my $groupmod = EBox::Global->modInstance('users');
+
+    my @existingGroups;
+    for my $group (@{$groups}) {
+        push (@existingGroups, $group) if ($groupmod->groupExists($group));
     }
 
     $self->set_list("printers/$printer/groups", "string", $groups);
@@ -1054,8 +1102,8 @@ sub _printerUsers # (printer)
     my ($self, $printer) = @_;
 
     unless ($self->dir_exists("printers/$printer")) {
-        throw EBox::Exceptions::DataNotFound('data' => 'printer',
-                'value' => $printer);
+        $self->_printerNotFound($printer);
+        return [];
     }
 
     return $self->get_list("printers/$printer/users");
@@ -1066,8 +1114,8 @@ sub _printerGroups # (group)
     my ($self, $printer) = @_;
 
     unless ($self->dir_exists("printers/$printer")) {
-        throw EBox::Exceptions::DataNotFound('data' => 'printer',
-                'value' => $printer);
+        $self->_printerNotFound($printer);
+        return [];
     }
 
     return $self->get_list("printers/$printer/groups");
@@ -1097,8 +1145,7 @@ sub _printersForGroup # (user)
     _checkGroupExists($group);
 
     my @printers;
-    for my $printer (@{$self->array_from_dir("printers")}) {
-        my $name = $printer->{'_dir'};
+    for my $name (@{$self->printers()}) {
         my $print = { 'name' => $name, 'allowed' => undef };
         my $groups = $self->get_list("printers/$name/groups");
         if (@{$groups}) {
@@ -1184,7 +1231,6 @@ sub setPrintersForGroup # (user, printers)
     }
 }
 
-
 sub delPrinter # (resource)
 {
     my ($self, $name) = @_;
@@ -1271,64 +1317,6 @@ sub enableQuota
     return (EBox::Config::configkey('enable_quota') eq 'yes');
 }
 
-# Method: currentUserQuota
-#
-#	Fetch the current set quota for a given user
-#
-# Parameters:
-#
-#	user - string
-#
-# Returns:
-#
-#	Integer - quota in MB. 0 means no quota
-#
-sub currentUserQuota
-{
-    my ($self, $user) = @_;
-
-    my $usermod = EBox::Global->modInstance('users');
-    unless (defined($user) and $usermod->uidExists($user)) {
-        throw EBox::Exceptions::External("user is not valid");
-    }
-    my @quotaValues = @{root(QUOTA_PROGRAM . " -q $user ")};
-
-    return $quotaValues[0];
-}
-
-# Method: setUserQuota
-#
-#	Set user quota
-#
-# Parameters:
-#
-#	Quota - Integer. Quota in MB. 0 no quota.
-#
-# Returns:
-#
-#	Integer - quota in MB. 0 means no quota
-#
-sub setUserQuota
-{
-    my ($self, $user, $userQuota) = @_;
-
-    return unless ($self->enableQuota());
-
-    my $usermod = EBox::Global->modInstance('users');
-    unless (defined($user) and $usermod->uidExists($user)) {
-        throw EBox::Exceptions::External("user is not valid");
-    }
-
-    unless (_checkQuota($userQuota)) {
-        throw EBox::Exceptions::InvalidData
-            ('data' => __('quota'), 'value' => $userQuota);
-    }
-
-    my $quota = $userQuota * 1024;
-
-    root(QUOTA_PROGRAM . " -s $user $quota");
-}
-
 sub dumpConfig
 {
     my ($self, $dir) = @_;
@@ -1340,21 +1328,65 @@ sub restoreConfig
 {
     my ($self, $dir) = @_;
 
-    $self->_loadSharesTree($dir);
-    $self->_fixLeftoverSharedDirectories();
-    $self->fixSIDs();
+    $self->set_bool('ignorePrinterNotFound', 1);
 
-    my $sambaLdapUser = new EBox::SambaLdapUser;
-    $sambaLdapUser->migrateUsers();
+    try {
+        $self->_loadSharesTree($dir);
+        $self->fixSIDs();
+
+        my $sambaLdapUser = new EBox::SambaLdapUser;
+        $sambaLdapUser->migrateUsers();
+    } finally {
+        $self->set_bool('ignorePrinterNotFound', 0);
+    }
 }
 
 sub restoreDependencies
 {
-    my ($self) = @_;
-    return ['users'];
+    my @depends = ();
+
+    push(@depends, 'users');
+
+    if ( EBox::Global->modExists('printers') )  {
+        push(@depends, 'printers');
+    }
+
+    return \@depends;
 }
 
 
+# backup domains
+
+sub backupDomains
+{
+    my $name = 'shares';
+    my %attrs  = (
+                  printableName => __('File Sharing'),
+                  description   => __(q{Shares, users and groups homes and profiles}),
+                 );
+
+    return ($name, \%attrs);
+}
+
+sub backupDomainsFileSelection
+{
+    my ($self, %enabled) = @_;
+    if ($enabled{shares}) {
+        my $sambaLdapUser = new EBox::SambaLdapUser();
+        my @dirs =  @{ $sambaLdapUser->sharedDirectories() };
+        push @dirs, map {
+            $_->{path}
+        } @{ $self->shares(1) };
+
+        my $selection = {
+                          includes => \@dirs,
+                         };
+        return $selection;
+    }
+
+    return {};
+}
+
 sub _dumpSharesTree
 {
     my ($self, $dir) = @_;
@@ -1396,18 +1428,18 @@ sub _loadSharesTree
         @shares = split '\s+', $contents;
     }
 
-
+    my @commands;
     foreach my $dirInfo (@shares) {
         my ($dir, $uid, $gid, $perm) = split ':', $dirInfo;
 
-        if (!-d $dir) {
-            EBox::Sudo::root("/bin/mkdir -p  '$dir'");
+        unless (-d $dir) {
+            push (@commands, "mkdir -p  '$dir'");
         }
 
-        EBox::Sudo::root("/bin/chmod $perm '$dir'"); # restore permissions
-        EBox::Sudo::root("/bin/chown $uid.$gid '$dir'");
-
+        push (@commands, "chmod $perm '$dir'"); # restore permissions
+        push (@commands, "chown $uid:$gid '$dir'");
     }
+    EBox::Sudo::root(@commands);
 }
 
 sub fixSIDs
@@ -1494,156 +1526,21 @@ sub _sharesTreeFile
 
 
 
-# we look for shared directories leftover from users and groups created
-# between a backup and a recovery. We move them to a leftover directories
-# so the data will be safe and retrevied by root
-sub _fixLeftoverSharedDirectories
-{
-    my ($self) = @_;
-
-
-    my @leftovers = $self->_findLeftoverSharedDirectories();
-    return if @leftovers == 0;
-
-    my $leftoversDir = $self->leftoversDir();
-
-    if (not EBox::Sudo::fileTest('-e', $leftoversDir)) {
-        EBox::Sudo::root("/bin/mkdir --mode=755 $leftoversDir");
-    }
-
-    my @leftoverTypes = qw(users groups);
-    foreach my $subdir (@leftoverTypes) {
-        if (not EBox::Sudo::fileTest('-e', "$leftoversDir/$subdir")) {
-            EBox::Sudo::root("/bin/mkdir --mode=755 $leftoversDir/$subdir");
-        }
-    }
-
-    foreach my $leftover (@leftovers) {
-        my $chownCommand = "/bin/chown root.root -R $leftover";
-        EBox::Sudo::root($chownCommand);
-
-        my $chmodDirCommand = "/bin/chmod 755 $leftover";
-        EBox::Sudo::root($chmodDirCommand);
-
-        # change permission to files in dir if dir has files
-        my $filesInDir = 1;
-        try {  EBox::Sudo::root("/bin/ls $leftover/*")  }  otherwise { $filesInDir = 0  } ;
-
-        if ($filesInDir) {
-            my $chmodFilesCommand = "  /bin/chmod -R og-srwx  $leftover/*";
-            EBox::Sudo::root($chmodFilesCommand);
-        }
-
-        my $leftoverNewDir =  $self->_leftoverNewDir($leftover, $leftoversDir);
-
-        my $mvCommand = "/bin/mv  $leftover $leftoverNewDir";
-        EBox::Sudo::root($mvCommand);
-        EBox::info("Moved leftover directory $leftover to $leftoverNewDir");
-    }
-}
-
-
-sub _leftoverNewDir
-{
-    my ($self, $leftover, $leftoversDir) = @_;
-
-    my $usersPath  = EBox::SambaLdapUser::usersPath();
-    my $groupsPath = EBox::SambaLdapUser::groupsPath();
-
-    my $leftoverType;
-    if ($leftover =~ m/^$usersPath/) {
-        $leftoverType = 'users/';
-    }
-    elsif ($leftover =~ m/^$groupsPath/) {
-        $leftoverType = 'groups/';
-    }
-    else {
-        EBox::warn("Can not determine the type of leftover $leftover; it will be store it in $leftoversDir");
-        $leftoverType = undef;
-    }
-
-    my $leftoverNewDir = "$leftoversDir/";
-    $leftoverNewDir .= $leftoverType if defined $leftoverType;  # better to store the leftover in a wrong place than lost it
-        $leftoverNewDir .= File::Basename::basename($leftover);
-
-    if (EBox::Sudo::fileTest('-e', $leftoverNewDir)) {
-        EBox::warn ("$leftoverNewDir already exists, we will choose another dir for this leftover. Please, remove or store away leftover directories" );
-        my $counter = 2;
-        my $oldLeftoverDir =$leftoverNewDir;
-        do  {
-            $leftoverNewDir = $oldLeftoverDir . ".$counter";
-            $counter = $counter +1 ;
-        } while (EBox::Sudo::fileTest('-e', $leftoverNewDir));
-        EBox::warn("The leftover will be stored in $leftoverNewDir");
-    }
-
-    return $leftoverNewDir;
-}
-
-
-sub _findLeftoverSharedDirectories
-{
-    my ($self) = @_;
-
-    my $sambaLdapUser = new EBox::SambaLdapUser;
-
-    my @leftovers;
-    my $sharedDirs = $sambaLdapUser->sharedDirectories();
-    return () if @{ $sharedDirs } == 0;
-
-
-    my $usersDir =  $sambaLdapUser->usersPath();
-    push @leftovers, $self->_findLeftoversInDir($usersDir, $sharedDirs);
-
-    my $groupsDir = $sambaLdapUser->groupsPath();
-    push @leftovers, $self->_findLeftoversInDir($groupsDir, $sharedDirs);
-
-    EBox::info("Leftovers shared directories found: @leftovers") if @leftovers > 0;
-    return @leftovers;
-}
-
-
-sub _findLeftoversInDir
-{
-    my ($self, $dir, $sharedDirs) = @_;
-    my $allShareDirs =  all(@{ $sharedDirs }) ;
-
-    my @candidateDirs;
-    try {
-        @candidateDirs = @{ EBox::Sudo::root("/usr/bin/find $dir/* -type d -maxdepth 0 ") };
-    }
-    catch EBox::Exceptions::Sudo::Command with { # we catch this because find will be fail if aren't any subdirectories in $dir
-        @candidateDirs = ();
-    };
-
-    chomp @candidateDirs;
-
-    my @leftovers = grep { $_ ne $allShareDirs  } @candidateDirs;
-    return @leftovers;
-}
-
-sub leftoversDir
-{
-    return EBox::SambaLdapUser::basePath() . '/leftovers';
-}
-
-
 # Overrides:
 #   EBox::Report::DiskUsageProvider::_facilitiesForDiskUsage
 sub _facilitiesForDiskUsage
 {
     my ($self) = @_;
 
-    my $usersPrintableName  = __(q{User's files});
+    my $usersPrintableName  = __(q{Users files});
     my $usersPath           = EBox::SambaLdapUser::usersPath();
-    my $groupsPrintableName = __(q{Group's files});
+    my $groupsPrintableName = __(q{Groups files});
     my $groupsPath          = EBox::SambaLdapUser::groupsPath();
 
     return {
         $usersPrintableName   => [ $usersPath ],
         $groupsPrintableName  => [ $groupsPath ],
     };
-
 }
 
 # Implement LogHelper interface
@@ -1741,9 +1638,19 @@ sub isAntivirusPresent
              and (-f '/usr/lib/samba/vfs/vscan-clamav.so'));
 }
 
+
+
+
+
+
+
 sub report
 {
     my ($self, $beg, $end, $options) = @_;
+    my $maxTopActivityUsers = $options->{'max_top_activity_users'};
+    my $maxTopActivityGroups =  $options->{'max_top_activity_groups'};
+    my $maxTopSizeShares =  $options->{'max_top_size_shares'};
+    my $maxTopVirusShares =  $options->{'max_top_virus_shares'};
 
     my $report = {};
 
@@ -1756,13 +1663,113 @@ sub report
         'select' => 'username, SUM(operations) AS operations',
         'from' => 'samba_access_report',
         'group' => 'username',
-        'limit' => $options->{'max_top_activity_users'},
+        'limit' => $maxTopActivityUsers,
         'order' => 'operations DESC'
     });
 
+
+    $report->{top_activity_groups} = _topActivityGroups($self, $beg, $end, $maxTopActivityGroups);
+
+    $report->{'top_size_shares'} = $self->runQuery($beg, $end, {
+        'select' => 'share, type, AVG(size) AS size',
+        'from' => 'samba_disk_usage_report',
+        'group' => 'share, type',
+        'limit' => $maxTopSizeShares,
+        'order' => 'size DESC',
+    });
+
+    $report->{'top_size_user_shares'} = $self->runQuery($beg, $end, {
+        'select' => 'share, AVG(size) AS size',
+        'from' => 'samba_disk_usage_report',
+        'group' => 'share',
+        'limit' => $maxTopSizeShares,
+        'order' => 'size DESC',
+        'where' => q{type = 'user'}
+    });
+
+    $report->{'top_size_group_shares'} = $self->runQuery($beg, $end, {
+        'select' => 'share, AVG(size) AS size',
+        'from' => 'samba_disk_usage_report',
+        'group' => 'share, type',
+        'limit' => $maxTopSizeShares,
+        'order' => 'size DESC',
+        'where' => q{type = 'group'}
+    });
+
+    $report->{'top_size_custom_shares'} = $self->runQuery($beg, $end, {
+        'select' => 'share, AVG(size) AS size',
+        'from' => 'samba_disk_usage_report',
+        'group' => 'share, type',
+        'limit' => $maxTopSizeShares,
+        'order' => 'size DESC',
+        'where' => q{type = 'custom'}
+    });
+
+    $report->{'top_virus_shares'} = $self->runQuery($beg, $end, {
+        'select' => 'share, SUM(virus) as virus',
+        'from' => 'samba_virus_share_report',
+        'group' => 'share',
+        'limit' => $maxTopVirusShares,
+        'order' => 'virus DESC',
+    });
+
+
     return $report;
 }
 
+
+sub _topActivityGroups
+{
+    my ($self, $beg, $end, $maxTopActivityGroups) = @_;
+    my $usersMod = EBox::Global->modInstance('users');
+
+    my $sqlResult = $self->runQuery($beg, $end, {
+        'select' => 'username, SUM(operations) AS operations',
+        'from' => 'samba_access_report',
+        'group' => 'username',
+    });
+    my %operationsByUser;
+    foreach my $i (0 .. $#{ $sqlResult->{username} } ) {
+        $operationsByUser{ $sqlResult->{username}->[$i] } =  $sqlResult->{operations}->[$i];
+    }
+
+    delete $sqlResult->{username};
+    delete $sqlResult->{operations};
+
+
+    my $min = -1;
+    my @groupsAndOperations;
+    foreach my $group_r ($usersMod->groups()) {
+        my $groupname = $group_r->{account};
+        my $operations = 0;
+
+        my @users = @{ $usersMod->usersInGroup($groupname) };
+        foreach my $user (@users) {
+            $operations += $operationsByUser{$user};
+        }
+
+
+        if (@groupsAndOperations <   $maxTopActivityGroups ) {
+            push @groupsAndOperations, [$groupname => $operations];
+        } elsif ($operations > $min) {
+            pop @groupsAndOperations;
+            push @groupsAndOperations, [$groupname => $operations];
+        } else {
+            next;
+        }
+
+        @groupsAndOperations = sort { $b->[1] <=> $a->[1]  } @groupsAndOperations;
+        $min = $groupsAndOperations[-1]->[1];
+    }
+
+    my @topGroups     = map { $_->[0]  } @groupsAndOperations;
+    my @topOperations = map { $_->[1] } @groupsAndOperations;
+    return {
+            'groupname' => \@topGroups,
+            'operations' => \@topOperations,
+           };
+}
+
 sub consolidateReportQueries
 {
     return [
@@ -1782,7 +1789,199 @@ sub consolidateReportQueries
                 'group' => 'client'
             }
         },
+        {
+            'target_table' => 'samba_disk_usage_report',
+            'query' => {
+                'select' => 'share, type, CAST (AVG(size) AS int) AS size',
+                'from' => 'samba_disk_usage',
+                'group' => 'share, type'
+
+            },
+        },
+        {
+            'target_table' => 'samba_virus_share_report',
+            'query' => {
+                'select' => 'filename, count(*) as virus',
+                'from' => 'samba_quarantine',
+                'where' => q{event='quarantine'},
+                'group' => 'filename'
+            },
+            'rowConversor' => \&_virusShareReportRowConversor,
+            'targetGroupFields' => ['share'],
+        },
     ];
 }
 
+
+
+
+
+my @sharesSortedByPathLen;
+
+sub _updatePathsByLen
+{
+    my ($self) = @_;
+
+    my $ldapInfo = EBox::SambaLdapUser->new();
+    @sharesSortedByPathLen = map {
+         { path => $_->{path},
+           share => $_->{sharename} }
+    } ( @{ $ldapInfo->userShareDirectories },
+        @{ $ldapInfo->groupShareDirectories }
+      );
+
+    foreach my $sh_r (@{ $self->shares(1) }) {
+        push @sharesSortedByPathLen, {path => $sh_r->{path},
+                                      share =>  $sh_r->{share} };
+    }
+
+    # add regexes
+    foreach my $share (@sharesSortedByPathLen) {
+        my $path = $share->{path};
+        $share->{pathRegex} = qr{^$path/};
+    }
+
+    @sharesSortedByPathLen = sort {
+        length($b->{path}) <=>  length($a->{path})
+    } @sharesSortedByPathLen;
+}
+
+sub _shareByFilename
+{
+    my ($filename) = @_;
+
+    if (not @sharesSortedByPathLen) {
+        my $samba =EBox::Global->modInstance('samba');
+        $samba->_updatePathsByLen();
+    }
+
+
+    foreach my $shareAndPath (@sharesSortedByPathLen) {
+        if ($filename =~ m/$shareAndPath->{pathRegex}/) {
+            return $shareAndPath->{share};
+        }
+    }
+
+
+    return undef;
+}
+
+
+sub _virusShareReportRowConversor
+{
+    my ($row) = @_;
+    my $filename = delete $row->{filename};
+    my $share = _shareByFilename($filename);
+    EBox::debug("COBV $filename -> $share");
+    if ($share) {
+        $row->{share} = $share;
+    } else {
+        return undef;
+    }
+
+
+    return $row;
+}
+
+
+sub logReportInfo
+{
+    my ($self) = @_;
+
+    if ($self->_diskUsageAlreadyCheckedToday) {
+        return [];
+    }
+
+    my @reportData;
+
+    my %shareByPath;
+
+    my @shares = @{ $self->_sharesAndSizes() };
+
+
+    foreach my $share (@shares) {
+        # add info about disk usage by share
+        my $entry = {
+                     table => 'samba_disk_usage',
+                     values => {
+                                share => $share->{share},
+                                size  => $share->{size},
+                                type  => $share->{type},
+                               }
+                    };
+        push @reportData, $entry;
+    }
+
+    return \@reportData;
+}
+
+
+sub _diskUsageAlreadyCheckedToday
+{
+    my ($self) = @_;
+
+    my $db = EBox::DBEngineFactory::DBEngine();
+    my $query = q{SELECT share FROM samba_disk_usage WHERE (timestamp >= (current_date + interval '0 day')) AND (timestamp < (current_date + interval '1 day'))};
+    my $results = $db->query($query);
+    return @{ $results } > 0;
+}
+
+sub _sharesAndSizes
+{
+    my ($self) = @_;
+
+    my $ldapInfo = EBox::SambaLdapUser->new();
+    my @shares;
+
+    foreach my $sh_r ( @{  $ldapInfo->userShareDirectories }) {
+        my $share = {
+                     share => $sh_r->{sharename},
+                     path  => $sh_r->{path},
+                     type  => 'user',
+                    };
+        push @shares, $share;
+    }
+
+    foreach my $sh_r ( @{  $ldapInfo->groupShareDirectories }) {
+        my $share = {
+                     share => $sh_r->{sharename},
+                     path  => $sh_r->{path},
+                     type  => 'group',
+                    };
+        push @shares, $share;
+    }
+
+
+
+    # add no-account shares to share list
+    foreach my $sh_r (@{ $self->shares(1)  }) {
+        my $share = {
+                     share => $sh_r->{share},
+                     path  => $sh_r->{path},
+                     type  => 'custom',
+                    };
+        push @shares, $share;
+    }
+
+
+    foreach my $share (@shares) {
+        my $path = $share->{path};
+        if (EBox::Sudo::fileTest('-d', $path)) {
+            my $output = EBox::Sudo::rootWithoutException("du -ms '$path'");
+
+            my ($size) =  $output->[0] =~ m{^(\d+)};
+            if (not defined $size) {
+                EBox::error("Problem getting $path size: @{$output}");
+                $size = 0;
+            }
+
+            $share->{size} = $size;
+        }
+    }
+
+
+    return \@shares;
+}
+
+
 1;
