@@ -1,4 +1,4 @@
-# Copyright (C) 2009 eBox Technologies S.L.
+# Copyright (C) 2009-2010 eBox Technologies S.L.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2, as
@@ -77,6 +77,24 @@ sub validateTypedRow
         throw EBox::Exceptions::External(
                 __('Netbios name cannot be longer than 15 characters'));
     }
+
+    # Check for incompatibility between PDC and PAM
+    # only on slave servers
+
+    my $users = EBox::Global->modInstance('users');
+    return unless ($users->mode() eq 'slave');
+
+    my $pdc = exists $newParams->{pdc} ?
+                  $newParams->{pdc}->value() :
+                  $oldParams->{pdc}->value();
+
+    my $pam = $users->model('PAM')->enable_pamValue();
+
+    if ($pam and $pdc) {
+        throw EBox::Exceptions::External(__x('A slave server can not act as PDC if PAM is enabled. You can do disable PAM at {ohref}LDAP Settings{chref}.',
+            ohref => q{<a href='/ebox/Users/Composite/Settings/'>},
+            chref => q{</a>}));
+    }
 }
 
 sub _table
@@ -92,7 +110,7 @@ sub _table
         new EBox::Types::DomainName(
             'fieldName' => 'workgroup',
             'printableName' => __('Domain name'),
-            'defaultValue' => 'EBOX',
+            'defaultValue' => 'ZENTYAL-DOMAIN',
             'editable' => 1,
         ),
         new EBox::Types::Text(
@@ -104,7 +122,7 @@ sub _table
         new EBox::Types::Text(
             'fieldName' => 'description',
             'printableName' => __('Description'),
-            'defaultValue' => 'EBox Samba Server',
+            'defaultValue' => 'Zentyal File Server',
             'editable' => 1,
         ),
         new EBox::Types::Union(
