@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Warp Networks S.L., DBS Servicios Informaticos S.L.
+# Copyright (C) 2008-2010 eBox Technologies S.L.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2, as
@@ -32,7 +32,6 @@ use EBox::Gettext;
 use Perl6::Junction qw(any all);
 use Error qw(:try);
 
-
 use Crypt::SmbHash qw(nthash ntlmgen);
 
 # Default values for samba user
@@ -44,15 +43,13 @@ use constant SMBPWDMUSTCHANGE   => '2147483647';
 use constant SMBGROUP           => '513';
 use constant SMBACCTFLAGS       => '[U]';
 use constant SMBACCTFLAGSDISABLED       => '[UD]';
-use constant GECOS              => 'Ebox file sharing user ';
 use constant USERGROUP          => 513;
-use constant DEFAULT_SHELL      => '/bin/false';
 # Home path for users and groups
 use constant BASEPATH           => '/home/samba';
-use constant USERSPATH          => BASEPATH . '/users';
+use constant USERSPATH          => '/home';
 use constant GROUPSPATH         => BASEPATH . '/groups';
 use constant PROFILESPATH       => BASEPATH . '/profiles';
-
+use constant QUOTA_PROGRAM => '/usr/share/ebox-samba/ebox-samba-quota';
 
 BEGIN
 {
@@ -64,7 +61,6 @@ BEGIN
     %EXPORT_TAGS = ( DEFAULT => \@EXPORT );
     @EXPORT_OK = qw();
     $VERSION = EBox::Config::version;
-
 }
 
 use base qw(EBox::LdapUserBase);
@@ -79,7 +75,6 @@ sub new
     return $self;
 }
 
-
 sub _smbHomes
 {
     my $samba = EBox::Global->modInstance('samba');
@@ -109,7 +104,6 @@ sub _domainUser
     return undef;
 }
 
-
 sub _userCommonLdapAttrs
 {
     my ($self) = @_;
@@ -134,8 +128,6 @@ sub _userCommonLdapAttrs
         sambaPwdLastSet      => time(),
 
         sambaAcctFlags       => $defaultFlags,
-
-        loginShell            => _loginShell(),
     };
 
     return $attrs;
@@ -178,12 +170,7 @@ sub _addUserLdapAttrs
                         sambaNTPassword      => $nt,
 
                         sambaSID             => $sambaSID,
-
-                        # gecos              => GECOS
                     ],
-                    replace => [
-                        homeDirectory => BASEPATH . "/users/$user",
-                    ]
             ]
         );
 
@@ -191,7 +178,7 @@ sub _addUserLdapAttrs
     }
     else {
         # upgrade from previous versions
-        # XXX currentl only the user common attributes are upgraded
+        # XXX currently only the user common attributes are upgraded
         my %searchParams = (base => $dn,
                             attrs => [keys %userCommonAttrs],
                             filter => "(objectclass=*)",
@@ -217,30 +204,34 @@ sub _addUserLdapAttrs
     }
 }
 
-
 # Implements LdapUserBase interface
 sub _addUser
 {
-    my ($self, $user, $password) = @_;
+    my ($self, $userName, $password, $quota) = @_;
 
     return unless ($self->{samba}->configured());
 
     my $users = EBox::Global->modInstance('users');
-    my $userInfo = $users->userInfo($user);
+    my $userInfo = $users->userInfo($userName);
     my $unixuid = $userInfo->{uid};
 
-    $self->_addUserLdapAttrs($user, $unixuid, $password);
+    $self->_addUserLdapAttrs($userName, $unixuid, $password);
 
     # Add user to Domain Users group
-    unless ($self->_domainUser($user)) {
-        $users->addUserToGroup($user, 'Domain Users');
+    unless ($self->_domainUser($userName)) {
+        $users->addUserToGroup($userName, 'Domain Users');
     }
 
-    my  $samba = EBox::Global->modInstance('samba');
-    $self->_createDir(USERSPATH . "/$user", $unixuid, USERGROUP, '0701');
-    $self->_createDir(PROFILESPATH . "/$user", $unixuid, USERGROUP, '0700');
-    $self->_createDir(PROFILESPATH . "/$user.V2", $unixuid, USERGROUP, '0700');
-    $self->{samba}->setUserQuota($unixuid, $samba->defaultUserQuota());
+    $self->_createDir(PROFILESPATH . "/$userName",
+        $unixuid, USERGROUP, '0700');
+    $self->_createDir(PROFILESPATH . "/$userName.V2",
+        $unixuid, USERGROUP, '0700');
+
+    unless (defined $quota and $quota >= 0) {
+        my $samba = EBox::Global->modInstance('samba');
+        $quota = $samba->defaultUserQuota();
+    }
+    $self->setUserQuota($userName, $quota);
 }
 
 sub _modifyUser
@@ -277,9 +268,6 @@ sub _delUser($$)
 
     return unless ($self->{samba}->configured());
 
-    if ( -d BASEPATH . "/users/$user") {
-        root ("rm -rf \'" .  BASEPATH . "/users/$user\'");
-    }
     if ( -d BASEPATH . "/profiles/$user") {
         root ("rm -rf \'" .  BASEPATH . "/profiles/$user\'");
     }
@@ -302,7 +290,7 @@ sub _delUserWarning
 
     settextdomain('ebox-samba');
     my $txt = __('This user has a sharing directory associated ' .
-                               'which contains data');
+                 'which contains data.');
     settextdomain('ebox-usersandgroups');
     unless ($self->_directoryEmpty($path)) {
         return ($txt);
@@ -319,8 +307,6 @@ sub _addGroup
     $self->addGroupLdapAttrs($group);
 }
 
-
-
 sub addGroupLdapAttrs
 {
     my ($self, $group, %params) = @_;
@@ -357,8 +343,6 @@ sub addGroupLdapAttrs
     }
 }
 
-
-
 sub setUserSID
 {
     my ($self, $user, $sid) = @_;
@@ -377,7 +361,6 @@ sub setUserSID
     $self->_setSID($dn, $sid);
 }
 
-
 sub setGroupSID
 {
     my ($self, $group, $sid) = @_;
@@ -396,7 +379,6 @@ sub setGroupSID
     $self->_setSID($dn, $sid);
 }
 
-
 sub _setSID
 {
     my ($self, $dn, $sambaSID) = @_;
@@ -411,6 +393,147 @@ sub _setSID
     $self->{ldap}->modify($dn, \%attrs);
 }
 
+sub _checkQuota # (quota)
+{
+    my ($quota) = @_;
+
+    ($quota =~ /\D/) and return undef;
+    return 1;
+}
+
+sub _setLdapQuota
+{
+    my ($self, $user, $quota) = @_;
+
+    my $userName = $user->{username};
+    my $usermod = EBox::Global->modInstance('users');
+    my $dn = "uid=$userName," . $usermod->usersDn();
+
+    if ($self->_isSambaObject('systemQuotas', $dn)) {
+        $self->{'ldap'}->modifyAttribute($dn, 'quota', $quota);
+    } else {
+        my %attrs = (
+            changes => [
+                add => [
+                    objectClass => 'systemQuotas',
+                    quota => $quota,
+                ],
+            ],
+        );
+        $self->{'ldap'}->modify($dn, \%attrs);
+    }
+
+}
+
+sub _setFilesystemQuota
+{
+    my ($self, $user, $userQuota) = @_;
+
+    my $quota = $userQuota * 1024;
+    my $uid = $user->{uid};
+    EBox::Sudo::root(QUOTA_PROGRAM . " -s $uid $quota");
+}
+
+# Method: setUserQuota
+#
+#	Set user quota
+#
+# Parameters:
+#
+#	Quota - Integer. Quota in MB. 0 no quota.
+#
+# Returns:
+#
+#	Integer - quota in MB. 0 means no quota
+#
+# TODO: Move everything about users quotas to users module
+sub setUserQuota
+{
+    my ($self, $userName, $userQuota) = @_;
+    return unless ($self->{samba}->enableQuota());
+
+    unless (_checkQuota($userQuota)) {
+        throw EBox::Exceptions::InvalidData
+            ('data' => __('quota'), 'value' => $userQuota);
+    }
+
+    my $usermod = EBox::Global->modInstance('users');
+    my $user = $usermod->userInfo($userName);
+
+    $self->_setLdapQuota($user, $userQuota);
+    $self->_setFilesystemQuota($user, $userQuota);
+}
+
+sub _currentLdapQuota
+{
+    my ($self, $user) = @_;
+
+    my $userName = $user->{username};
+    my $usermod = EBox::Global->modInstance('users');
+    my $dn = "uid=$userName," . $usermod->usersDn();
+
+    if ($self->_isSambaObject('systemQuotas', $dn)) {
+        # TODO when moving quotas to users and groups module, quota information
+        # will be in userInfo($user)
+        my %attrs = (
+            base => $usermod->usersDn(),
+            filter => "(uid=$userName)",
+            scope => 'one',
+            attrs => ['quota'],
+        );
+        my $result = $self->{'ldap'}->search(\%attrs);
+        my $entry = $result->entry(0);
+        my $quota = $entry->get_value('quota');
+        if ($quota) {
+            return $quota;
+        }
+    }
+    throw EBox::Exceptions::Internal("Quota not found in LDAP for $userName");
+}
+
+sub _currentFilesystemQuota
+{
+    my ($self, $user) = @_;
+
+    my $uid = $user->{uid};
+    my @quotaValues = @{EBox::Sudo::root(QUOTA_PROGRAM . " -q $uid ")};
+    return $quotaValues[0];
+}
+
+# Method: currentUserQuota
+#
+#	Fetch the current set quota for a given user
+#
+# Parameters:
+#
+#	user - string
+#
+# Returns:
+#
+#	Integer - quota in MB. 0 means no quota
+#
+# TODO: Move everything about users quotas to users module
+sub currentUserQuota
+{
+    my ($self, $userName) = @_;
+
+    my $usermod = EBox::Global->modInstance('users');
+    my $user = $usermod->userInfo($userName);
+
+    my $filesystemQuota = $self->_currentFilesystemQuota($user);
+
+    try {
+        my $ldapQuota = $self->_currentLdapQuota($user);
+
+        if ($filesystemQuota != $ldapQuota) {
+            $self->_setFilesystemQuota($user, $ldapQuota);
+        }
+        return $ldapQuota;
+    } catch EBox::Exceptions::Internal with {
+        return $filesystemQuota;
+    }
+}
+
 sub _delGroup
 {
     my ($self, $group) = @_;
@@ -435,7 +558,7 @@ sub _delGroupWarning
     my $path = BASEPATH . "/groups/$group";
     settextdomain('ebox-samba');
     my $txt = __('This group has a sharing directory associated ' .
-                 'that contains data');
+                 'that contains data.');
     settextdomain('ebox-usersandgroups');
 
     unless ($self->_directoryEmpty($path)) {
@@ -452,12 +575,15 @@ sub _userAddOns
     return unless ($self->{samba}->configured());
 
     my $users = EBox::Global->modInstance('users');
-
     my $samba = EBox::Global->modInstance('samba');
-    my $uid = $users->userInfo($username)->{'uid'};
+
     my @args;
     my $share = $self->_userSharing($username) ? "yes" : "no";
     my $printers = $samba->_printersForUser($username);
+    my $quota = $self->currentUserQuota($username);
+    if ($quota < 0) {
+        $quota = 0;
+    }
     my $args =  {
         'username' => $username,
         'share'    => $share,
@@ -466,7 +592,7 @@ sub _userAddOns
 
         'printers' => $printers,
         'printerService' => $samba->printerService,
-        'quota' => $samba->currentUserQuota($uid)
+        'quota' => $quota,
     };
 
     return { path => '/samba/samba.mas', params => $args };
@@ -480,7 +606,6 @@ sub _groupAddOns
 
     my $samba = EBox::Global->modInstance('samba');
 
-    use Data::Dumper;
     my @args;
     my $share = $self->_groupSharing($groupname) ? "yes" : "no";
     my $printers = $samba->_printersForGroup($groupname);
@@ -501,7 +626,7 @@ sub _createDir
 {
     my ($self, $path, $uid, $gid, $chmod) = @_;
 
-    if (-d $path) {
+    if (EBox::Sudo::fileTest('-d', $path)) {
         return;
     }
 
@@ -523,7 +648,6 @@ sub _directoryEmpty
     return ($#ent == 1);
 }
 
-
 sub migrateUsers
 {
     my ($self) = @_;
@@ -534,7 +658,8 @@ sub migrateUsers
     # update users
     foreach my $user ($users->users) {
         my $username = $user->{'username'};
-        $self->_addUser($username);
+        my $quota = $self->currentUserQuota($username);
+        $self->_addUser($username, undef, $quota);
 
         # check if we have old administrator rights
         try {
@@ -560,7 +685,6 @@ sub migrateUsers
     }
 }
 
-
 sub checkDomainSID
 {
     my ($sid) = @_;
@@ -575,7 +699,6 @@ sub checkDomainSID
     return 1;
 }
 
-
 sub checkSID
 {
     # we do not discrimintae now etween domainSID and a element SID
@@ -595,6 +718,7 @@ sub getSID
 
     my $domain = $samba->workgroup();
 
+    # FIXME wtf
     my $res = `sudo net getlocalsid $domain 2>&1 `;
     if ($? != 0) {
         # return undef;
@@ -615,7 +739,6 @@ sub getSID
     return $sid;
 }
 
-
 sub alwaysGetSID
 {
     my $sid;
@@ -627,11 +750,9 @@ sub alwaysGetSID
 sub generateSID
 {
     # FIXME: Hardcore SID for testing purposes
-    #
     return 'S-1-5-21-3818554400-921237426-3143208535';
 }
 
-
 sub getGroupSID
 {
     my ($self, $group) = @_;
@@ -684,6 +805,36 @@ sub _sharingResourceExists
     return ($result->count == 1)
 }
 
+
+# TODO Find another name more self-explanatory, this one is crap
+sub userShareDirectories
+{
+    my ($self) = @_;
+
+    my $ldap  = $self->{ldap};
+    my $users = EBox::Global->modInstance('users');
+
+    my $dn =  $users->usersDn;
+    my %attrs = (
+        base   => $dn,
+        filter => '(&(objectclass=posixAccount) (objectclass=sambaSamAccount))',
+        attrs  => [ 'uid', 'homedirectory'],
+        scope  => 'one'
+            );
+    my $result = $ldap->search(\%attrs);
+
+    my @share;
+    foreach my $entry ($result->entries) {
+        my $user = $entry->get_value('uid');
+        my $home = $entry->get_value('homeDirectory');
+        push (@share, { path      => $home,
+                        username => $user,
+                        sharename => $user
+                  });
+    }
+    return \@share;
+}
+
 # TODO Find another name more self-explanatory, this one is  crap
 sub groupShareDirectories
 {
@@ -730,7 +881,7 @@ sub sharingName
     my %attrs = (
             base   => $dn,
             filter => "(objectclass=eboxGroup)",
-            attrs  => [ 'displayResource'],
+            attrs  => ['displayResource'],
             scope  => 'base'
             );
 
@@ -742,7 +893,6 @@ sub sharingName
     return $value ? $value : "";
 }
 
-
 # Sets the name for a sharing resource in a group
 sub setSharingName
 {
@@ -756,7 +906,7 @@ sub setSharingName
     }
 
     if ((not defined $name) or ( $name =~ /^\s*$/)) {
-        throw EBox::Exceptions::External(__("A name should be provided for the share"));
+        throw EBox::Exceptions::External(__("A name should be provided for the share."));
     }
 
     my $oldname = $self->sharingName($group);
@@ -789,12 +939,10 @@ sub setSharingName
         my $uid = getpwnam(EBox::Config::user);
         my $gid = $users->groupGid($group);
         $self->_createDir(BASEPATH . "/groups/$group",
-                          $uid, $gid, "+t,g+w");
+                          $uid, $gid, "g+w");
     }
 }
 
-
-
 sub removeSharingName
 {
     my ($self, $group) = @_;
@@ -817,8 +965,6 @@ sub removeSharingName
     $global->modChange('samba');
 }
 
-
-
 sub _pwdChanged
 {
     my ($self, $entry, $newPassword) = @_;
@@ -828,7 +974,6 @@ sub _pwdChanged
     return ($ntpwd ne nthash($newPassword));
 }
 
-
 sub _getAccountFlags
 {
     my ($self, $username) = @_;
@@ -914,7 +1059,6 @@ sub sambaDomainName
     }
 }
 
-
 # Method: setSambaDomainName
 #
 #   Set the samba domain name. The entry is created if it does not
@@ -951,8 +1095,8 @@ sub setSambaDomainName
             ]
         );
 
-	my $dn = "sambaDomainName=$domain," . $ldap->dn();
-	$ldap->add($dn, \%attrs);
+    my $dn = "sambaDomainName=$domain," . $ldap->dn();
+    $ldap->add($dn, \%attrs);
 }
 
 sub _fetchDomainAttrs
@@ -1025,7 +1169,6 @@ sub deleteSambaDomainNameAttrs
 #   }
 }
 
-
 sub deleteSambaDomains
 {
     my ($self) = @_;
@@ -1043,7 +1186,6 @@ sub deleteSambaDomains
     }
 }
 
-
 # Method: setSambaDomain
 #
 #   Set the samba domain name. The entry is created if it does not
@@ -1059,9 +1201,6 @@ sub deleteSambaDomains
 # sub setSambaDomains
 # {
 #   my ($self) = @_;
-
-
-
 #   my $users = EBox::Global->modInstance('users');
 #   my $lastUid = $users->lastUid;
 #   my $lastGid = $users->lastGid;
@@ -1071,24 +1210,18 @@ sub deleteSambaDomains
 #                   gidNumber => $lastGid,
 #                      },
 #              };
-
 #   my $ldap = $self->{ldap};
 #   my %attrs = (
 #               base => $ldap->dn,
 #               filter => "objectClass=sambaDomain",
 #               scope => "sub"
 #           );
-
-
 #   my @sambaDomains = $ldap->search(\%attrs)->entries();
 #   if (not @sambaDomains) {
 #     return 0;
 #   }
-
-
 #   foreach my $entry (@sambaDomains) {
 #     my $dn = $entry->dn;
-
 #     if (not $ldap->isObjectClass($dn, 'sambaUnixIdPool')) {
 #       $ldap->modify($dn,
 #             { add => {
@@ -1097,22 +1230,15 @@ sub deleteSambaDomains
 #                   gidNumber => $lastGid,
 #                  }
 #             }
-
 #            );
 #     }
 #     else {
 #       $ldap->modify($dn, $modifyParams);
 #     }
-
-
 #   }
-
 #   return 1;
 # }
 
-
-
-
 # Method: updateNetbiosName
 #
 #   Update in LDAP all those attributes which contain the netbios name
@@ -1162,7 +1288,6 @@ sub updateNetbiosName
 #       that depending on when the user/group is created the SID might change.
 #       This method should be run in regenConfig
 #
-#
 sub updateSIDEntries
 {
     my ($self) = @_;
@@ -1219,7 +1344,8 @@ sub updateSIDEntries
     }
 }
 
-
+# TODO: It may be useful in other ldap modules, move to users package, quota
+# code depends on it and it also has to be moved
 sub _isSambaObject
 {
     my ($self, $object, $dn) = @_;
@@ -1263,7 +1389,6 @@ sub sharedDirectories
     return \@dirs;
 }
 
-
 sub basePath
 {
     return BASEPATH;
@@ -1274,27 +1399,23 @@ sub  usersPath
     return USERSPATH;
 }
 
-
 sub groupsPath
 {
     return GROUPSPATH;
 }
 
-sub _loginShell
-{
-    my $shell = EBox::Config::configkey('login_shell');
-
-    if (defined($shell)) {
-        return $shell;
-    } else {
-        return DEFAULT_SHELL;
-    }
-}
-
 sub schemas
 {
-    return [ EBox::Config::share() . "ebox-samba/samba.ldif",
-             EBox::Config::share() . "ebox-samba/ebox.ldif" ];
+    return [
+        EBox::Config::share() . "ebox-samba/samba.ldif",
+        EBox::Config::share() . "ebox-samba/ebox.ldif",
+        EBox::Config::share() . "ebox-samba/quota.ldif",
+    ];
+}
+
+sub indexes
+{
+    return ['sambaSID', 'sambaGroupType', 'sambaSIDList', 'sambaDomainName'];
 }
 
 sub acls
